@@ -96,7 +96,7 @@ fn read_template_values_from_definitions(
                 |cap| {
                     let key = cap.get(1).unwrap().as_str().to_string();
                     let value = cap.get(2).unwrap().as_str().to_string();
-                    println!("{} => '{}'", key, value);
+                    println!("{key} => '{value}'");
                     template_values.insert(key, Value::from(value));
                     Ok(template_values)
                 },
